@@ -219,7 +219,6 @@ hypre_CSRMatrixAddFirstPass( HYPRE_Int              firstrow,
 HYPRE_Int
 hypre_CSRMatrixAddSecondPass( HYPRE_Int          firstrow,
                               HYPRE_Int          lastrow,
-                              HYPRE_Int         *twspace,
                               HYPRE_Int         *marker,
                               HYPRE_Int         *map_A2C,
                               HYPRE_Int         *map_B2C,
@@ -431,7 +430,7 @@ hypre_CSRMatrixAddHost ( HYPRE_Complex    alpha,
                                   A, B, nrows_A, nnzrows_C, ncols_A, rownnz_C,
                                   memory_location_C, C_i, &C);
 
-      hypre_CSRMatrixAddSecondPass(ns, ne, twspace, marker, NULL, NULL,
+      hypre_CSRMatrixAddSecondPass(ns, ne, marker, NULL, NULL,
                                    rownnz_C, alpha, beta, A, B, C);
 
       hypre_TFree(marker, HYPRE_MEMORY_HOST);
@@ -1303,15 +1302,16 @@ hypre_CSRMatrixTranspose(hypre_CSRMatrix  *A,
  *--------------------------------------------------------------------------*/
 
 /* RL: TODO add memory locations */
-HYPRE_Int hypre_CSRMatrixSplit(hypre_CSRMatrix  *Bs_ext,
-                               HYPRE_BigInt      first_col_diag_B,
-                               HYPRE_BigInt      last_col_diag_B,
-                               HYPRE_Int         num_cols_offd_B,
-                               HYPRE_BigInt     *col_map_offd_B,
-                               HYPRE_Int        *num_cols_offd_C_ptr,
-                               HYPRE_BigInt    **col_map_offd_C_ptr,
-                               hypre_CSRMatrix **Bext_diag_ptr,
-                               hypre_CSRMatrix **Bext_offd_ptr)
+HYPRE_Int
+hypre_CSRMatrixSplit(hypre_CSRMatrix  *Bs_ext,
+                     HYPRE_BigInt      first_col_diag_B,
+                     HYPRE_BigInt      last_col_diag_B,
+                     HYPRE_Int         num_cols_offd_B,
+                     HYPRE_BigInt     *col_map_offd_B,
+                     HYPRE_Int        *num_cols_offd_C_ptr,
+                     HYPRE_BigInt    **col_map_offd_C_ptr,
+                     hypre_CSRMatrix **Bext_diag_ptr,
+                     hypre_CSRMatrix **Bext_offd_ptr)
 {
    HYPRE_Complex   *Bs_ext_data = hypre_CSRMatrixData(Bs_ext);
    HYPRE_Int       *Bs_ext_i    = hypre_CSRMatrixI(Bs_ext);
@@ -1324,10 +1324,11 @@ HYPRE_Int hypre_CSRMatrixSplit(hypre_CSRMatrix  *Bs_ext,
    HYPRE_Complex   *B_ext_diag_data = NULL;
    HYPRE_Int       *B_ext_offd_i = NULL;
    HYPRE_Int       *B_ext_offd_j = NULL;
+   HYPRE_BigInt    *B_ext_offd_bigj = NULL;
    HYPRE_Complex   *B_ext_offd_data = NULL;
    HYPRE_Int       *my_diag_array;
    HYPRE_Int       *my_offd_array;
-   HYPRE_BigInt    *temp;
+   HYPRE_BigInt    *temp = NULL;
    HYPRE_Int        max_num_threads;
    HYPRE_Int        cnt = 0;
    hypre_CSRMatrix *Bext_diag = NULL;
@@ -1408,19 +1409,15 @@ HYPRE_Int hypre_CSRMatrixSplit(hypre_CSRMatrix  *Bs_ext,
          B_ext_diag_i[num_rows_Bext] = B_ext_diag_size;
          B_ext_offd_i[num_rows_Bext] = B_ext_offd_size;
 
-         if (B_ext_diag_size)
-         {
-            B_ext_diag_j    = hypre_CTAlloc(HYPRE_Int,     B_ext_diag_size, HYPRE_MEMORY_HOST);
-            B_ext_diag_data = hypre_CTAlloc(HYPRE_Complex, B_ext_diag_size, HYPRE_MEMORY_HOST);
-         }
-         if (B_ext_offd_size)
-         {
-            B_ext_offd_j    = hypre_CTAlloc(HYPRE_Int,     B_ext_offd_size, HYPRE_MEMORY_HOST);
-            B_ext_offd_data = hypre_CTAlloc(HYPRE_Complex, B_ext_offd_size, HYPRE_MEMORY_HOST);
-         }
+         B_ext_diag_j    = hypre_CTAlloc(HYPRE_Int,     B_ext_diag_size, HYPRE_MEMORY_HOST);
+         B_ext_diag_data = hypre_CTAlloc(HYPRE_Complex, B_ext_diag_size, HYPRE_MEMORY_HOST);
+         B_ext_offd_j    = hypre_CTAlloc(HYPRE_Int,     B_ext_offd_size, HYPRE_MEMORY_HOST);
+         B_ext_offd_bigj = hypre_CTAlloc(HYPRE_BigInt,  B_ext_offd_size, HYPRE_MEMORY_HOST);
+         B_ext_offd_data = hypre_CTAlloc(HYPRE_Complex, B_ext_offd_size, HYPRE_MEMORY_HOST);
          if (B_ext_offd_size || num_cols_offd_B)
          {
-            temp = hypre_CTAlloc(HYPRE_BigInt, B_ext_offd_size + num_cols_offd_B, HYPRE_MEMORY_HOST);
+            temp = hypre_CTAlloc(HYPRE_BigInt, B_ext_offd_size + num_cols_offd_B,
+                                 HYPRE_MEMORY_HOST);
          }
       }
 
@@ -1437,12 +1434,12 @@ HYPRE_Int hypre_CSRMatrixSplit(hypre_CSRMatrix  *Bs_ext,
             if (Bs_ext_j[j] < first_col_diag_B || Bs_ext_j[j] > last_col_diag_B)
             {
                temp[cnt_offd] = Bs_ext_j[j];
-               B_ext_offd_j[cnt_offd] = Bs_ext_j[j];
+               B_ext_offd_bigj[cnt_offd] = Bs_ext_j[j];
                B_ext_offd_data[cnt_offd++] = Bs_ext_data[j];
             }
             else
             {
-               B_ext_diag_j[cnt_diag] = Bs_ext_j[j] - first_col_diag_B;
+               B_ext_diag_j[cnt_diag] = (HYPRE_Int) (Bs_ext_j[j] - first_col_diag_B);
                B_ext_diag_data[cnt_diag++] = Bs_ext_data[j];
             }
          }
@@ -1500,15 +1497,19 @@ HYPRE_Int hypre_CSRMatrixSplit(hypre_CSRMatrix  *Bs_ext,
       {
          for (j = B_ext_offd_i[i]; j < B_ext_offd_i[i + 1]; j++)
          {
-            B_ext_offd_j[j] = hypre_BigBinarySearch(col_map_offd_C, B_ext_offd_j[j], num_cols_offd_C);
+            B_ext_offd_j[j] = hypre_BigBinarySearch(col_map_offd_C,
+                                                    B_ext_offd_bigj[j],
+                                                    num_cols_offd_C);
          }
       }
    } /* end parallel region */
 
    hypre_TFree(my_diag_array, HYPRE_MEMORY_HOST);
    hypre_TFree(my_offd_array, HYPRE_MEMORY_HOST);
+   hypre_TFree(B_ext_offd_bigj, HYPRE_MEMORY_HOST);
 
-   Bext_diag = hypre_CSRMatrixCreate(num_rows_Bext, last_col_diag_B - first_col_diag_B + 1,
+   Bext_diag = hypre_CSRMatrixCreate(num_rows_Bext,
+                                     (HYPRE_Int) (last_col_diag_B - first_col_diag_B + 1),
                                      B_ext_diag_size);
    hypre_CSRMatrixMemoryLocation(Bext_diag) = HYPRE_MEMORY_HOST;
    Bext_offd = hypre_CSRMatrixCreate(num_rows_Bext, num_cols_offd_C, B_ext_offd_size);
@@ -1914,7 +1915,7 @@ hypre_CSRMatrixComputeRowSum( hypre_CSRMatrix *A,
  *      4: abs diag inverse sqrt
  *--------------------------------------------------------------------------*/
 
-void
+HYPRE_Int
 hypre_CSRMatrixExtractDiagonalHost( hypre_CSRMatrix *A,
                                     HYPRE_Complex   *d,
                                     HYPRE_Int        type)
@@ -1925,6 +1926,7 @@ hypre_CSRMatrixExtractDiagonalHost( hypre_CSRMatrix *A,
    HYPRE_Int     *A_j    = hypre_CSRMatrixJ(A);
    HYPRE_Int      i, j;
    HYPRE_Complex  d_i;
+   char           msg[HYPRE_MAX_MSG_LEN];
 
    for (i = 0; i < nrows; i++)
    {
@@ -1941,23 +1943,33 @@ hypre_CSRMatrixExtractDiagonalHost( hypre_CSRMatrix *A,
             {
                d_i = hypre_cabs(A_data[j]);
             }
-            else if (type == 2)
+            else
             {
-               d_i = 1.0 / (A_data[j]);
-            }
-            else if (type == 3)
-            {
-               d_i = 1.0 / (hypre_sqrt(A_data[j]));
-            }
-            else if (type == 4)
-            {
-               d_i = 1.0 / (hypre_sqrt(hypre_cabs(A_data[j])));
+               if (A_data[j] == 0.0)
+               {
+                  hypre_sprintf(msg, "Zero diagonal found at row %i!", i);
+                  hypre_error_w_msg(HYPRE_ERROR_GENERIC, msg);
+               }
+               else if (type == 2)
+               {
+                  d_i = 1.0 / A_data[j];
+               }
+               else if (type == 3)
+               {
+                  d_i = 1.0 / hypre_sqrt(A_data[j]);
+               }
+               else if (type == 4)
+               {
+                  d_i = 1.0 / hypre_sqrt(hypre_cabs(A_data[j]));
+               }
             }
             break;
          }
       }
       d[i] = d_i;
    }
+
+   return hypre_error_flag;
 }
 
 /*--------------------------------------------------------------------------
@@ -1969,7 +1981,7 @@ hypre_CSRMatrixExtractDiagonalHost( hypre_CSRMatrix *A,
  *      3: diag inverse sqrt
  *--------------------------------------------------------------------------*/
 
-void
+HYPRE_Int
 hypre_CSRMatrixExtractDiagonal( hypre_CSRMatrix *A,
                                 HYPRE_Complex   *d,
                                 HYPRE_Int        type)
@@ -1986,6 +1998,8 @@ hypre_CSRMatrixExtractDiagonal( hypre_CSRMatrix *A,
    {
       hypre_CSRMatrixExtractDiagonalHost(A, d, type);
    }
+
+   return hypre_error_flag;
 }
 
 /*--------------------------------------------------------------------------
